use psxcdgen_ex::{encoder::psx::{calculate_psx_lbas, encode_psx_image}, file_writer::{ImageType, write_image}, types::{layout::Layout, CDDesc, File, Directory}};
use std::{path::PathBuf, str::FromStr};
use tool_helper::Error;

fn make_file(name: &str) -> Result<File, Error> {
    let mut name = name.to_owned();

    name.push_str(".txt");
    File::new_regular(name.as_ref(), "Planschbecken sind planschig und so".to_owned().into_bytes())
}

fn populate() -> Result<CDDesc, Error> {
    let mut desc   = CDDesc::new();
    let     file   = make_file("Planschi")?;
    let     file2  = make_file("Wuff")?;
    let     folder = {
        let mut folder     = Directory::new("Sub")?;
        let     sub_folder = {
            let mut folder = Directory::new("SubSub")?;

            folder.add_file(make_file("Blubb")?);
            folder
        };

        folder.add_dir(sub_folder);
        folder.add_file(make_file("Schwimm")?);
        folder.add_file(make_file("Miau")?);
        folder
    };

    desc.add_dir(folder);
    desc.add_file(file);
    desc.add_file(file2);

    calculate_psx_lbas(&mut desc);
    Ok(desc)
}

fn run_main() -> Result<(), Error> {
    let desc = populate()?;

    println!("\n<== Planschbecken ==>");
    for element in desc.get_memory_layout().iter() {
        match element {
            Layout::SystemArea(_)  => println!("SystemArea:"),
            Layout::PVD(_)         => println!("PVD:"),
            Layout::PathTables(_)  => println!("PathTables:"),
            Layout::Directory(dir) => println!("Dir: {} @{}-{}", dir.borrow().name, dir.borrow().get_track_rel_lba(), dir.borrow().get_extended_size()),
            Layout::File(file)     => println!("File: {} @{}-{}", file.borrow(), file.borrow().get_track_rel_lba(), file.borrow().get_extended_size()),
        }
    }
    println!("\n<== Planschbecken ==>");
    for rand_item in desc.create_file_map() {
        println!("{}", rand_item.0);
    }

    println!("\n<== Planschbecken ==>\nStart encoding");
    write_image(desc, encode_psx_image, ImageType::BinCue, PathBuf::from_str("planschi.bin")?)
}

fn main() {
    match run_main() {
        Ok(_)      => println!("\n<== Planschbecken ==>"),
        Err(error) => println!("{}", error)
    }
}